/*
 *  DielectricMaterial.cpp
 *  Assignment5
 *
 *  Created by Zach Gildersleeve on 2/20/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "DielectricMaterial.h"
#include <stdio.h>

DielectricMaterial::DielectricMaterial(double newEta, int newPhongTerm) : eta(newEta), phongTerm(newPhongTerm) {
	inverseEta = 1.0 / eta;
	r0 = (eta - 1.0) / (eta + 1.0);
	r0 = r0 * r0;
	beersE = Color(0.0, 0.0, 0.0);
}

DielectricMaterial::DielectricMaterial(double newEta, int newPhongTerm, const Color &newBeersE) : eta(newEta), phongTerm(newPhongTerm) {
	inverseEta = 1.0 / eta;
	r0 = (eta - 1.0) / (eta + 1.0);
	r0 = r0 * r0;
	beersE = Color(log(newBeersE.r), log(newBeersE.g), log(newBeersE.b));
}

void DielectricMaterial::shade(Color &result, const RenderContext &context,
							   const Ray &ray, const HitRecord &hit, int depth,
							   const Color &attenuation) const {
	
	// P = O + tV
	Vector rayDirection = ray.getDirection();
	Point hitPosition = ray.getOrigin() + (rayDirection * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	Vector unflippedNormal = normal;
	double etaTemp = 0.0;
	bool exiting;
	
	//costheta = N dot V
	double cosTheta = dot(normal, rayDirection);
		
	if (cosTheta > 0)	// exiting surface
	{
		etaTemp = inverseEta;
		normal = -normal;
		exiting = true;
	}
	else				// entering surface
	{
		etaTemp = eta;
		cosTheta = -cosTheta;
		exiting = false;
	}
	Scene *myScene = context.scene;	

	double cosTheta2sq = 1.0 + (cosTheta * cosTheta - 1.0) / (etaTemp * etaTemp);
	Vector reflectDirection = rayDirection + normal * (2.0 * cosTheta);
//	Point hitP = hitPosition - (normal * 1.0e-4);
	reflectDirection.normalize();

	
	if (cosTheta2sq < 0)
	{
		// total internal reflection
		// trace reflection ray like metal
		Ray reflectRay(hitPosition, reflectDirection);
		Color reflectedColor;
		double currentT = myScene->traceRay(reflectedColor, context, reflectRay, depth + 1, attenuation, hit.getPrimitive());
		
		// test for entering or exiting
		if(exiting)
			result = reflectedColor * beersAttenuation(currentT);
		else
			result = reflectedColor;		
	}
	else
	{
		double cosTheta2 = sqrt(cosTheta2sq);
		double cosM = min(cosTheta, cosTheta2);
		double sch5 = pow(1.0 - cosM, 5);
	
		double Fr = r0 + (1.0 - r0) * sch5;

		Ray reflectRay(hitPosition, reflectDirection);
		Color reflectedColor;
		double currentT = myScene->traceRay(reflectedColor, context, reflectRay, depth + 1, attenuation * Fr);
		
		// test for entering or exiting
		if (exiting)
			result = reflectedColor * Fr * beersAttenuation(currentT);
		else
			result = reflectedColor * Fr;			

	// cast transparent ray
		Vector transparentDirection = rayDirection * (1.0 / etaTemp) + normal * ((cosTheta / etaTemp) - cosTheta2);
	//	transparentDirection.normalize();
		Color transparentColor;
		Ray transparentRay(hitPosition, transparentDirection);
		currentT = myScene->traceRay(transparentColor, context, transparentRay, depth + 1, attenuation * (1.0 - Fr));
		
		if (!exiting)
			result = transparentColor * (1.0 - Fr) * beersAttenuation(currentT);
		else
			result += transparentColor * (1.0 - Fr);
	}

	const Object *world = myScene->object;
	Color currentSpecColor(0, 0, 0);

	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
		 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(unflippedNormal, Lnormal);
		
		if (!exiting && cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive())
			{
				//			light += currentLightColor * Kdiffuse * cosPhi;
				Vector H;
				if(exiting)
					H = Lnormal + rayDirection;
				else
					H = Lnormal - rayDirection;
					
				H.normalize();
				
				double cosAlpha = dot(H, unflippedNormal);
				if (cosAlpha > 0)
				{
					double p = pow(cosAlpha, phongTerm);
					currentSpecColor += currentLightColor * p;
				}				
			}
		}
	}
	result += currentSpecColor;
}
