/*
 *  Scene.cpp
 *  Assignment2
 *
 *  Created by Zach Gildersleeve on 1/28/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Scene.h"
#include <iostream>
#include <float.h>



Scene::Scene() {
	finalColor = Color(0.0, 0.0, 0.0);
}

Scene::~Scene() {
	delete background;
	delete camera;
	delete object;
	delete image;
	lightList.erase(lightList.begin(), lightList.end());
	//delete lightList;
}

void Scene::preprocess() {
	background->preprocess();
	double ratio = image->getAspectRatio();
	camera->preprocess(ratio);
	object->preprocess();
	
	//std::vector<Object *>::const_iterator i;
	
	for (int i = 0; i < lightList.size(); i++)
	{
		Light *currentLight = lightList[i];
		currentLight->preprocess();
	}
	
	xRes = image->getWidth();
	yRes = image->getHeight();
}

void Scene::render() {

	RenderContext context(this);
	Color result;
	Color initialColorAttenuation(1.0, 1.0, 1.0);

	double xMult = (xRes / 2.0) + 0.5;
	double yMult = (yRes / 2.0) + 0.5;
	double xInv = 2.0 / xRes;
	double yInv = 2.0 / yRes;

	for(int i=0;i<yRes;i++){
	
	if (i % 16 == 0)
		std::cout << "\tRendering Line" << i << "\n";

		double y = (i - yMult) * yInv; 
				
		for(int j=0;j<xRes;j++){ 

			double x = (j - xMult) * xInv; 

			Ray ray;
			HitRecord hit(FLT_MAX);	
//			std::cout << "making ray" << i << " " << j << "\n";
			camera->makeRay(ray, context, x, y);
			object->intersect(hit, context, ray);
			
			if(!hit.getPrimitive())
			{
				background->getBackgroundColor(result, context, ray);
			}
			else
			{
				hit.getMaterial()->shade(result, context, ray, hit, 0, initialColorAttenuation);			
			}

			image->set(j, i, result); 
		} 
	} 
}

void Scene::render(int newX, int newY) {
	
	RenderContext context(this);
	Color result; 
	
	double xMult = (xRes / 2.0) + 0.5;
	double yMult = (yRes / 2.0) + 0.5;
	double xInv = 2.0 / xRes;
	double yInv = 2.0 / yRes;
			
	double y = (newY - yMult) * yInv; 
	double x = (newX - xMult) * xInv; 
			
	Ray ray;
	HitRecord hit(FLT_MAX);
	Color initialColorAttenuation(1.0, 1.0, 1.0);
	
	camera->makeRay(ray, context, x, y);
	object->intersect(hit, context, ray);
	
	if(!hit.getPrimitive())
	{
		background->getBackgroundColor(result, context, ray);
	}
	else
	{
		hit.getMaterial()->shade(result, context, ray, hit, 0, initialColorAttenuation);			
	}
	
	image->setPixel(newX, newY, result);
}

double Scene::traceRay(Color &result, const RenderContext &context, const Ray &ray,
					 int depth, const Color &attenuatedColor) const {
	
	if (depth > maxRayDepth || attenuatedColor.getMax() < minAttenuation)
	{
		result = finalColor;
		return 0;
	}
	else {
		HitRecord hit(FLT_MAX);	
		object->intersect(hit, context, ray);
		
		if(hit.getPrimitive())
		{
			hit.getMaterial()->shade(result, context, ray, hit, depth, attenuatedColor);
			return hit.getT();	

		}
		else
		{
			background->getBackgroundColor(result, context, ray);
			return DBL_MAX;
		}
	}
}
	
double Scene::traceRay(Color &result, const RenderContext &context, const Ray &ray,
					int depth, const Color &attenuatedColor, const Object *insideObject) const {
						 
	 if (depth > maxRayDepth || attenuatedColor.getMax() < minAttenuation)
	 {
		 result = finalColor;
		 return 0;
	 }
	 else {
		 HitRecord hit(FLT_MAX);	
		 insideObject->intersect(hit, context, ray);
		 
		 if(hit.getPrimitive())
		 {
			 hit.getMaterial()->shade(result, context, ray, hit, depth, attenuatedColor);
			 return hit.getT();	
		 }
		 else
		 {
			 background->getBackgroundColor(result, context, ray);
			 return DBL_MAX;	
		 }
	 }
}

