/*
 *  Vector.h
 *  Assignment1
 *
 *  Created by Zach Gildersleeve on 1/14/07.
 *
 */

#ifndef VECTOR_H
#define VECTOR_H

#include "Math.h"

class Point;

class Vector {
	public:	
		double x, y, z;
		                  
		Vector() {}
		inline Vector(const Vector &a) : x(a.x), y(a.y), z(a.z) {}
		inline Vector(double newX, double newY, double newZ): x(newX), y(newY), z(newZ) {}
		
		~Vector() {}
		
		inline Vector &operator=(const Vector &a) {
			x = a.x;
			y = a.y;
			z = a.z;
			return *this;	
		}
		
		bool operator==(const Vector &a) {
			return x == a.z && y == a.y && z == a.z;
		}
		
		bool operator!=(const Vector &a) {
			return x != a.x || y != a.y || z != a.z;
		}
		
		Vector operator-() const {
			return Vector(-x, -y, -z);
		}
		
		Vector operator+(const Vector &b) const {
			return Vector(x + b.x, y + b.y, z + b.z);
		}
		
		Vector operator-(const Vector &b) const {
			return Vector(x - b.x, y - b.y, z - b.z);
		}
		
		Vector operator*(const Vector &a) const {
			return Vector(x * a.x, y * a.y, z * a.z);
		}
		
		Vector operator*(double s) const {
			return Vector(x * s, y * s, z * s);
		}
		
		Vector operator/(const Vector &a) const {
			return Vector(x / a.x, y / a.y, z / a.z);
		}
		
		Vector operator/(double s) const {
			double recip = 1.0 / s;
			return Vector(x * recip, y * recip, z * recip);
		}
				
		Vector &operator+=(const Vector &a) {
			x += a.x;
			y += a.y;
			z += a.z;
			return *this;
		}
		Vector &operator-=(const Vector &a) {
			x -= a.x;
			y -= a.y;
			z -= a.z;
			return *this;
		}
		Vector &operator*=(const Vector &a) {
			x *= a.x;
			y *= a.y;
			z *= a.z;
			return *this;
		}
		Vector &operator*=(double s) {
			x *= s;
			y *= s;
			z *= s;
			return *this;
		}
		Vector &operator/=(const double s) {
			double inverse = 1.0 / s;
			x *= inverse;
			y *= inverse;
			z *= inverse;
			return *this;
		}
		
		double length() const {
			return (double)(sqrt(x*x + y*y + z*z));
		}
		
		double lengthSquared() const {
			return (double)(x*x + y*y + z*z);
		}
		
		double normalize() {
			double length = this->length();
//			if (length > 0)
//			{
				double lengthRecip = 1.0 / length;
				x *= lengthRecip;
				y *= lengthRecip;
				z *= lengthRecip;
//			}
			return length;
		}
		
		Vector inverse() const {
			return Vector(1.0 / x, 1.0 / y, 1.0 / z);
		}
					
		// need to be defined outside Vector.h
		inline explicit Vector(const Point &p);
};

#include "Point.h"

inline Vector::Vector(const Point &p) {
	x = p.x;
	y = p.y;
	z = p.z;
}

inline double dot(const Vector &a, const Vector &b) {
	return (a.x*b.x + a.y*b.y + a.z*b.z);
}

inline Vector cross(const Vector a, const Vector b) {
	return Vector(a.y*b.z - a.z*b.y,
				  a.z*b.x - a.x*b.z,
				  a.x*b.y - a.y*b.x);
}

inline Vector minVector(const Vector &p1, const Vector &p2)
{
	return Vector(min(p1.x, p2.x), min(p1.y, p2.y), min(p1.z, p2.z));
}

inline Vector maxVector(const Vector &p1, const Vector &p2)
{
	return Vector(max(p1.x, p2.x), max(p1.y, p2.y), max(p1.z, p2.z));
}

#endif